# Copyright(c) 2005-2011 Association of Universities for Research in Astronomy, Inc.
#
# CKINPUT -- Check input and output and return a list.
#
# Version: Sept 2, 2005 FV
#          May 2,  2008, NZ   Copy from nifs$nfcheck to this new name

procedure ckinput () 

string  input       {"", prompt = "Input files to check"}
string  output      {"", prompt = "Output files to check"}
string  prefix      {"", prompt = "Output prefix"}
file    outlist     {"", prompt = "Return list of files"}
string  name        {"CKINPUT", prompt = "Task name"}
string  dependflag  {"*PREPAR*", prompt = "Dependent processing flag"}
string  procflag    {"", prompt = "Previous processing flag"}
string  sci_ext     {"SCI", prompt = "Science extension"}
file    logfile     {"STDOUT", prompt = "Logfile"}
bool    verbose     {yes, prompt = "Verbose?"}
bool    vverbose    {no, prompt = "Very verbose?"}

int     nfiles      {prompt = "Number of files"}
int     nver        {prompt = "Number of extension versions"}
int     status      {prompt = "Return status"}

struct  *scanfile1  {"", prompt="Internal use only"}
struct  *scanfile2  {"", prompt="Internal use only"}

begin
    bool    runbefore
    int     nbad, naxis, junk
    file    img, phu
    file    tmpfile, tmpphu, tmpsci, tmpin, tmpout
    struct  line

    # Define temporary files
    tmpfile = mktemp ("tmpfile") 
    tmpphu = mktemp ("tmpphu") 
    tmpsci = mktemp ("tmpsci") 
    tmpin = mktemp ("tmpin") 
    tmpout = mktemp ("tmpout") 

    # Cache parameter files
    cache ("gemextn")

    # Initialize
    status = 1
    nver = INDEF

    # Check for output list.
    if (access (outlist)) {
        printf ("ERROR - %s: Output list %s exists.\n", name, outlist) | \
            scan (line)
        printlog (line, logfile, verbose+)
        goto clean
    }

    # Expand and verify input
    if (vverbose) print ("expansion and verification of input")
    gemextn (input, check="mef,exists", process="none", index="",
        extname="", extversion="", ikparams="", omit="extension, kernel",
        replace="", outfile=tmpfile, logfile="", glogpars="",
        verbose=verbose)
    if (gemextn.count == 0 || gemextn.fail_count != 0) {
        printf ("ERROR - %s: Problem with input.\n", name) | scan (line)
        printlog (line, logfile, verbose+)
        goto clean
    }
    nfiles = gemextn.count

    # Check PHUs
    gemextn ("@" // tmpfile, check="mef,exists", process="append",
        index="0", extname="", extversion="", ikparams="",
        omit="extension, kernel", replace="", outfile=tmpphu, 
        logfile="", glogpars="", verbose=verbose)
    if (gemextn.count != nfiles || gemextn.fail_count != 0) {
        printf ("ERROR - %s: Problem with PHUs.\n", name) | scan (line)
        printlog (line, logfile, verbose+)
        goto clean
    }

    # Make input list of images not previously run and check keywords.
    scanfile1 = tmpphu
    scanfile2 = tmpfile
    nfiles = 0
    nbad = 0
    while (fscan (scanfile1, phu) != EOF) {
        junk = fscan (scanfile2, img)
        if (vverbose) print ("file: " // phu)
        runbefore = no

        # Check previous processing flag.
        if (procflag != "") {
            line = ""
            hselect (phu, procflag, yes) | scan (line)
            if (line != "") {
                printf ("WARNING - %s: Image %s has been run\n",
                    name, img) | scan (line)
                printlog (line, logfile, verbose+)
                printf ("                       through %s before.\n",
                    name) | scan (line)
                printlog (line, logfile, verbose+)
                runbefore = yes
            }
        }

        # Check dependent flag.
        if (dependflag != "") {
            line = ""
            hselect(phu, dependflag, yes) | scan(line)
            if (line == "") {
                printf ("WARNING - %s: Image %s has not been prepared (%s).\n",
                    name, img, dependflag) | scan (line)
                printlog (line, logfile, verbose+) 
                nbad += 1
            }
        }

        # Add to list of not previously run input files.
        if (!runbefore) {
            print (img, >> tmpin)
            nfiles += 1
        }
    }

    # Check science extensions.
    gemextn ("@" // tmpfile, check="mef,exists", process="expand",
        index="", extname=sci_ext, extversion="1-", ikparams="",
        omit="extension,index", replace="", outfile=tmpsci,
        logfile="", glogpars="", verbose=verbose)
    if (gemextn.count == 0 || gemextn.fail_count != 0) {
        printf ("ERROR - %s:  Problems with science data.\n", name) | \
            scan (line)
        printlog (line, logfile, verbose+)
        goto clean
    }

    # Check dimensions.
    scanfile1 = tmpsci
    while (fscan (scanfile1, img) != EOF) {
        hselect (img, "NAXIS", yes) | scan (naxis)
        if (nscan() != 1) {
            printf ("WARNING - %s: Data in %s missing NAXIS.\n",
                name, img) | scan (line)
            printlog (line, logfile, verbose+)
            nbad += 1
        } else if (naxis != 2) {
            printf ("WARNING - %s: Data in %s are not 2-D.\n", name, img) | \
                scan (line)
            printlog (line, logfile, verbose+)
            nbad += 1
        }
    }

    # Check for empty file list
    if (nfiles == 0) {
        printf ("ERROR - %s:  No input images to process.\n", name) | \
            scan (line)
        printlog (line, logfile, verbose+)
        goto clean
    }

    # Exit if problems found with input files
    if (nbad > 0) {
        printf ("ERROR - %s: %d bad image(s)\n", name, nbad) | scan (line)
        printlog (line, logfile, verbose+)
        goto clean
    }

    # Check that all images have the same number of SCI extensions.
    if (vverbose) print ("checking version numbers")
    scanfile1 = tmpin
    while (fscan (scanfile1, img) != EOF) {
        if (isindef(nver)) {
            gemextn (img, check="exists", process="expand", index="",
                extname=sci_ext, extversion="1-",
                ikparams="", omit="", replace="", outfile="dev$null",
                logfile="", glogpars="", verbose-)
            nver = gemextn.count
        } else
            gemextn (img, check="exists", process="expand", index="",
                extname=sci_ext, extversion="1-" // nver,
                ikparams="", omit="", replace="", outfile="dev$null",
                logfile="", glogpars="", verbose-)
        if (nver == 0 || gemextn.count != nver || gemextn.fail_count != 0) {
            printf ("ERROR - %s: Bad or missing science data in %s.\n",
                name, img) | scan (line)
            printlog (line, logfile, verbose+)
            goto clean
        }
    }

    # Log input files.
    printlog ("Using input files:", logfile, verbose) 
    if (verbose) type (tmpin) 
    type (tmpin, >> logfile) 

    # Expand and verify output
    if (output != "" || prefix != "") {
        if (vverbose) print ("expansion and verification of output")
        gemextn (output, check="absent", process="none", index="",
            extname="", extversion="", ikparams="",
            omit="extension,kernel", replace="", outfile=tmpout,
            logfile="", glogpars="", verbose=verbose)
        if (gemextn.fail_count != 0) {
            printf ("ERROR - %s: Problems with output.\n", name) | scan (line)
            printlog (line, logfile, verbose+)
            goto clean
        }
        if (gemextn.count == 0) {
            printf ("%%^%%%s%%@%s\n", prefix, tmpin) | scan (line)
            gemextn (line, check="absent", process="none", index="",
                extname="", extversion="", ikparams="",
                omit="kernel,exten", replace="", outfile=tmpout,
                logfile="", glogpars="", verbose=verbose)
            if (gemextn.count == 0 || gemextn.fail_count != 0) {
                printf ("ERROR - %s: No or incorrectly formatted output \
                    files\n", name) | scan (line)
                printlog (line, logfile, verbose+)
                goto clean
            }
        }

        # Check number of input and output images
        if (nfiles != gemextn.count) {
            printf ("ERROR - %s: Different number of input and output files\n",
                name) | scan (line)
            printlog (line, logfile, verbose+)
            goto clean
        }

        # Log output files.
        printlog ("Using output files:", logfile, verbose) 
        if (verbose) type (tmpout) 
        type (tmpout, >> logfile) 

        if (vverbose) print ("constructing file list")
        joinlines (tmpin, tmpout, output=outlist, delim=" ", \
            missing="Missing", maxchar=161, shortest=yes, verbose=no)
    } else
        rename (tmpin, outlist)

    # Completed successfully
    status = 0

clean:
    scanfile1 = ""
    scanfile2 = ""
    delete (tmpfile, ver-, >& "dev$null") 
    delete (tmpphu, ver-, >& "dev$null") 
    delete (tmpsci, ver-, >& "dev$null") 
    delete (tmpin, ver-, >& "dev$null") 
    delete (tmpout, ver-, >& "dev$null") 
end
